import dateutil.parser

from datetime import datetime, date
from dateutil.tz import tz


class TimeWarriorInterval(object):
    def __init__(self, start, end, tags, annotation):
        self.__start = self.__get_local_datetime(start)
        self.__end = self.__get_local_datetime(end) if end is not None else None
        self.__tags = tags
        self.__annotation = annotation

    def __eq__(self, other):
        return self.__start == other.get_start() \
            and self.__end == other.get_end() \
            and self.__tags == other.get_tags() \
            and self.__annotation == other.get_annotation()

    def __hash__(self):
        return hash(repr(self))

    def get_start(self):
        return self.__start

    def get_end(self):
        return self.__end

    def get_tags(self):
        return self.__tags

    def get_annotation(self):
        return self.__annotation

    def is_open(self):
        return self.__end is None

    def get_duration(self):
        if self.is_open():
            return datetime.now(tz=tz.tzlocal()) - self.__start
        else:
            return self.__end - self.__start

    def get_date(self):
        return date(self.__start.year, self.__start.month, self.__start.day)

    def __get_local_datetime(self, datetime_string):
        from_zone = tz.tzutc()
        to_zone = tz.tzlocal()

        date = dateutil.parser.parse(datetime_string)
        date.replace(tzinfo=from_zone)

        return date.astimezone(to_zone)



import json
import re

from timewreport.config import TimeWarriorConfig
from timewreport.interval import TimeWarriorInterval


class TimeWarriorParser(object):
    def __init__(self, input_stream):
        self.__config = self.__parse_configuration_section(input_stream)
        self.__intervals = self.__parse_intervals_section(input_stream)

    @staticmethod
    def __parse_configuration_section(input_stream):
        config = {}

        for line in input_stream:
            if line == u'\x0A':
                break

            m = re.search('^([^:]+):( (.*))?$', line, re.MULTILINE)
            config[m.group(1)] = m.group(3) if m.group(2) is not None else ""

        return TimeWarriorConfig(config)

    @staticmethod
    def __parse_intervals_section(input_stream):
        json_string = ''

        for line in input_stream:
            json_string += line

        intervals = []

        for interval in json.loads(json_string):
            intervals.append(TimeWarriorInterval(
                interval['start'],
                interval['end'] if 'end' in interval else None,
                interval['tags'] if 'tags' in interval else [],
                interval['annotation'] if 'annotation' in interval else None
            ))

        return intervals

    def get_config(self):
        return self.__config

    def get_intervals(self):
        return self.__intervals



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

config = {
    'name': 'timew-report',
    'version': '1.3.1',
    'description': 'An interface for TimeWarrior report data',
    'long_description': long_description,
    'long_description_content_type': 'text/markdown',
    'url': 'https://github.com/lauft/timew-report.git',
    'author': 'Thomas Lauf',
    'author_email': 'Thomas.Lauf@tngtech.com',
    'license': 'MIT License',
    'classifiers': [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.4',
    ],
    'keywords': 'timewarrior taskwarrior time-tracking',
    'packages': ['timewreport'],
    'install_requires': ['python-dateutil'],
}

setup(**config)



import re


class TimeWarriorConfig(object):
    def __init__(self, config=None):
        self.__config = config if config is not None else {}

    def update(self, other):
        if isinstance(other, TimeWarriorConfig):
            config = other.get_dict()
        elif isinstance(other, dict):
            config = other
        else:
            raise TypeError()

        self.__config.update(config)

    def get_dict(self):
        return self.__config

    def get_value(self, key, default):
        if key in self.__config:
            return self.__config[key]
        else:
            return default

    def get_boolean(self, key, default):
        value = self.get_value(key, default)

        return True if re.search('^(on|1|yes|y|true)$', '{}'.format(value), re.IGNORECASE) else False

    def get_int(self, key, default):
        value = self.get_value(key, default)

        return int(value) if value is not None else None

    def get_debug(self):
        return self.get_boolean('debug', False)

    def get_verbose(self):
        return self.get_boolean('verbose', False)

    def get_confirmation(self):
        return self.get_boolean('confirmation', False)
